import { Button, Checkbox, Input, Space } from 'antd'
import React, { useEffect, useState } from 'react'
import { useLocation, useNavigate } from 'react-router-dom'
import styled from 'styled-components'
import { getOnceJoblog } from '../services/index'
import { UpOutlined, DownOutlined } from '@ant-design/icons'
import Mark from 'mark.js'

const LogWrapper = styled.div`
  margin: 20px;
  padding: 30px;
  background-color: #fff;
  .log {
    background-color: #012b36;
    height: calc(100vh - 200px);
    overflow-y: scroll;
    color: #638691;
    padding: 8px 10px;
    ::-webkit-scrollbar {
      display: none;
    }
  }
  .return {
    display: flex;
    justify-content: space-between;
    padding-bottom: 10px;
  }
`

let timeout = null

const LogWatcher = () => {
  const navigate = useNavigate()
  const { state } = useLocation()
  const [logData, setLogData] = useState(null)
  const [autoRefresh, setAutoRefresh] = useState(false)

  const [keyWord, setKeyWord] = useState(null)

  const [keyWordList, setKeyWordList] = useState([])

  const [keyWordIndex, setKeyWordIndex] = useState(0)

  const fetchOnceJoblog = async () => {
    const { data } = await getOnceJoblog(state.id)
    if (data.code === 200) {
      setLogData(data.data.log)
      if (![2, 3].includes(data.data.status) && autoRefresh) {
        timeout = setTimeout(() => {
          fetchOnceJoblog()
        }, 5000)
      }
    }
  }
  useEffect(() => {
    fetchOnceJoblog()
  }, [state])

  const returnLogTable = () => {
    navigate('/datasource', {
      state: {
        key: '3',
        currentPage: state.currentPage,
        currentPageSize: state.currentPageSize,
        logId: state.logId,
      },
    })
  }

  const changeRefresh = e => {
    if (e.target.checked) {
      fetchOnceJoblog()
    } else {
      clearTimeout(timeout)
    }
    setAutoRefresh(e.target.checked)
  }

  const onClickRefresh = () => {
    fetchOnceJoblog()
  }

  const findLast = () => {
    if (keyWordList.length) {
      const logWindow = document.getElementById('log')
      keyWordList[keyWordIndex - 1].style.backgroundColor = '#fffd38'
      if (keyWordIndex - 1 > 0) {
        setKeyWordIndex(keyWordIndex - 1)
        keyWordList[keyWordIndex - 2].style.backgroundColor = '#fd9640'
        const targetbox =
          document.getElementsByClassName('keyword_mark')[keyWordIndex - 2]
        logWindow.scrollTop = targetbox.offsetTop - 150
        logWindow.scrollLeft = targetbox.offsetLeft - logWindow.clientWidth
      } else {
        setKeyWordIndex(keyWordList.length)
        keyWordList[keyWordList.length - 1].style.backgroundColor = '#fd9640'
        const targetbox =
          document.getElementsByClassName('keyword_mark')[
            keyWordList.length - 1
          ]
        logWindow.scrollTop = targetbox.offsetTop - 150
        logWindow.scrollLeft = targetbox.offsetLeft - logWindow.clientWidth
      }
    }
  }
  const findNext = () => {
    if (keyWordList.length) {
      const logWindow = document.getElementById('log')
      keyWordList[keyWordIndex - 1].style.backgroundColor = '#fffd38'
      if (keyWordIndex + 1 <= keyWordList.length) {
        setKeyWordIndex(keyWordIndex + 1)
        keyWordList[keyWordIndex].style.backgroundColor = '#fd9640'
        const targetbox =
          document.getElementsByClassName('keyword_mark')[keyWordIndex]
        logWindow.scrollTop = targetbox.offsetTop - 150
        logWindow.scrollLeft = targetbox.offsetLeft - logWindow.clientWidth
      } else {
        setKeyWordIndex(1)
        keyWordList[0].style.backgroundColor = '#fd9640'
        const targetbox = document.getElementsByClassName('keyword_mark')[0]
        logWindow.scrollTop = targetbox.offsetTop - 150
        logWindow.scrollLeft = targetbox.offsetLeft - logWindow.clientWidth
      }
    }
  }
  const changeKeyWord = e => {
    setKeyWord(e.target.value)
    setKeyWordIndex(0)
    const markInstance = new Mark(document.querySelector('#log'))
    markInstance.unmark({
      done: () => {
        const list = []
        markInstance.mark(e.target.value, {
          each: val => {
            val.style.backgroundColor = '#fffd38'
            val.style.padding = 0
            val.className = 'keyword_mark'
            list.push(val)
          },
        })
        if (list.length > 0) {
          setKeyWordIndex(1)
          list[0].style.backgroundColor = '#fd9640'
        }
        setKeyWordList(list)
      },
    })
  }
  const searchTools = (
    <Space>
      <span>
        {keyWordIndex}/{keyWordList.length}
      </span>
      <UpOutlined onClick={findLast} />
      <DownOutlined onClick={findNext} />
    </Space>
  )

  return (
    <LogWrapper>
      <div className="return">
        <Space>
          <Input
            value={keyWord}
            onChange={changeKeyWord}
            addonAfter={searchTools}
            placeholder="输入关键词"
          />
        </Space>
        <Space>
          <Checkbox onChange={changeRefresh} checked={autoRefresh}>
            自动刷新
          </Checkbox>
          <Button onClick={onClickRefresh}>刷新</Button>
          <Button
            type="primary"
            onClick={() => {
              returnLogTable()
            }}>
            返回上一级
          </Button>
        </Space>
      </div>
      <pre className="log" id="log">
        {logData}
      </pre>
    </LogWrapper>
  )
}

export default LogWatcher
